package ru.landgrafhomyak.serdha.api.v0.ddl

import ru.landgrafhomyak.serdha.api.v0.Expression
import ru.landgrafhomyak.serdha.api.v0.LowLevelApi

public interface TableCreator<TableUserExtension : Any> {
	public val expressionBuilder: Expression.Builder<TableUserExtension>

	public val types: ColumnType.Builder

	public fun <RuntimeType, DatabaseType : ColumnType<RuntimeType>> column(name: String, type: DatabaseType): Column<RuntimeType, DatabaseType, TableUserExtension>

	public fun index(name: String, vararg columns: Column<*, *, TableUserExtension>): Index<TableUserExtension>

	public fun uniqueIndex(name: String, distinctNulls: Boolean, vararg columns: Column<*, *, TableUserExtension>): UniqueIndex<TableUserExtension>
	public fun checkConstraint(name: String, constraint: Expression<Boolean, ColumnType.BOOLEAN, TableUserExtension>): CheckConstraint<TableUserExtension>
	public fun <RuntimeType : Any, DatabaseType : ColumnType<RuntimeType>> defaultValue(c: Column<RuntimeType, DatabaseType, TableUserExtension>, expr: Expression<RuntimeType, DatabaseType, TableUserExtension>): DefaultConstraint<RuntimeType, DatabaseType, TableUserExtension>

	@Suppress("PropertyName")
	public val rowId_column: Column<RowId<TableUserExtension>, ColumnType<RowId<TableUserExtension>>, TableUserExtension>

	@Suppress("PropertyName")
	public val rowId_uniqueConstraint: UniqueIndex<TableUserExtension>

	@Suppress("FunctionName")
	@LowLevelApi
	public fun <TargetTableUserWrapper : Any> _foreignKey(
		fromColumns: Array<Column<*, *, TableUserExtension>>,
		toTable: Table<TargetTableUserWrapper, *>,
		toColumns: Array<Column<*, *, TargetTableUserWrapper>>,
		onUpdate: ForeignKey.OnUpdateAction,
		onDelete: ForeignKey.OnDeleteAction,
	): ForeignKey<TableUserExtension, TargetTableUserWrapper>

	@OptIn(LowLevelApi::class)
	public fun <TargetTableUserWrapper : Any, ColumnType1 : ColumnType<*>> foreignKey(
		fromColumn: Column<*, ColumnType1, TableUserExtension>,
		toTable: Table<TargetTableUserWrapper, *>,
		toColumn: Column<*, ColumnType1, TargetTableUserWrapper>,
		onUpdate: ForeignKey.OnUpdateAction,
		onDelete: ForeignKey.OnDeleteAction,
	): ForeignKey<TableUserExtension, TargetTableUserWrapper> = this._foreignKey(
		fromColumns = arrayOf(fromColumn),
		toTable = toTable,
		toColumns = arrayOf(toColumn),
		onUpdate = onUpdate,
		onDelete = onDelete
	)

	@OptIn(LowLevelApi::class)
	public fun <TargetTableUserWrapper : Any, ColumnType1 : ColumnType<*>, ColumnType2 : ColumnType<*>> foreignKey(
		fromColumn1: Column<*, ColumnType1, TableUserExtension>,
		fromColumn2: Column<*, ColumnType2, TableUserExtension>,
		toTable: Table<TargetTableUserWrapper, *>,
		toColumn1: Column<*, ColumnType1, TargetTableUserWrapper>,
		toColumn2: Column<*, ColumnType2, TargetTableUserWrapper>,
		onUpdate: ForeignKey.OnUpdateAction,
		onDelete: ForeignKey.OnDeleteAction,
	): ForeignKey<TableUserExtension, TargetTableUserWrapper> = this._foreignKey(
		fromColumns = arrayOf(fromColumn1, fromColumn2),
		toTable = toTable,
		toColumns = arrayOf(toColumn1, toColumn2),
		onUpdate = onUpdate,
		onDelete = onDelete
	)

	@OptIn(LowLevelApi::class)
	public fun <TargetTableUserWrapper : Any, ColumnType1 : ColumnType<*>, ColumnType2 : ColumnType<*>, ColumnType3 : ColumnType<*>> foreignKey(
		fromColumn1: Column<*, ColumnType1, TableUserExtension>,
		fromColumn2: Column<*, ColumnType2, TableUserExtension>,
		fromColumn3: Column<*, ColumnType3, TableUserExtension>,
		toTable: Table<TargetTableUserWrapper, *>,
		toColumn1: Column<*, ColumnType1, TargetTableUserWrapper>,
		toColumn2: Column<*, ColumnType2, TargetTableUserWrapper>,
		toColumn3: Column<*, ColumnType3, TargetTableUserWrapper>,
		onUpdate: ForeignKey.OnUpdateAction,
		onDelete: ForeignKey.OnDeleteAction,
	): ForeignKey<TableUserExtension, TargetTableUserWrapper> = this._foreignKey(
		fromColumns = arrayOf(fromColumn1, fromColumn2, fromColumn3),
		toTable = toTable,
		toColumns = arrayOf(toColumn1, toColumn2, toColumn3),
		onUpdate = onUpdate,
		onDelete = onDelete
	)
}